@@ -36,7 +36,7 @@ class MFELandmarking:
        type, via kwargs argument of ``extract`` method of MFE class.
 
     4. The return value of all feature extraction methods should be a single
-       value or a generic Sequence (preferably a :obj:`np.ndarray`) type with
+       value or a generic List (preferably a :obj:`np.ndarray`) type with
        numeric values.
 
     There is another type of method adopted for automatic detection. It is
@@ -910,7 +910,7 @@ class MFELandmarking:
 
         model = sklearn.neighbors.KNeighborsClassifier(
             n_neighbors=1,
-            algorithm="auto",
+            algorithm="kd_tree",
             weights="uniform",
             p=2,
             metric="minkowski",
