@@ -1,32 +1,38 @@
 """Main module for extracting metafeatures from datasets.
 """
 import typing as t
-import collections
 import shutil
 import time
+import warnings
 
 import texttable
 import numpy as np
+import pandas as pd
 import sklearn.utils
 import sklearn.exceptions
+import tqdm.auto
 
 import pymfe._internal as _internal
+import pymfe._bootstrap as _bootstrap
 
-_TypeSeqExt = t.Sequence[
+_TypeSeqExt = t.List[
     t.Tuple[str, t.Callable, t.Tuple[str, ...], t.Tuple[str, ...]]
 ]
 """Type annotation for a sequence of TypeExtMtdTuple objects."""
 
+_TypeExtract = t.Union[t.Tuple[t.List, ...], t.Dict[str, t.List], pd.DataFrame]
+"""Type annotation for the possible output types of the extract."""
+
 
 class MFE:
     """Core class for metafeature extraction.
 
     Attributes
     ----------
-    X : :obj:`Sequence`
+    X : :obj:`List`
         Independent attributes of the dataset.
 
-    y : :obj:`Sequence`
+    y : :obj:`List`
         Target attributes of the dataset.
 
     groups : :obj:`tuple` of :obj:`str`
@@ -38,9 +44,10 @@ class MFE:
         meta-feature extraction, from selected metafeatures groups and features
         listed at instantiation.
 
-        summary : :obj:`tuple` of :obj:`str`
-            Tuple object which contains summary functions names for features
-            summarization.
+    summary : :obj:`tuple` of :obj:`str`
+        Tuple object which contains summary functions names for features
+        summarization.
+
     """
 
     groups_alias = [("default", _internal.DEFAULT_GROUP)]
@@ -264,14 +271,14 @@ class MFE:
             groups=self.groups,
             suppress_warnings=suppress_warnings,
             wildcard=wildcard,
-        )  # type: t.Tuple[t.Tuple[str, ...], _TypeSeqExt, t.Tuple[str, ...]]
+        )
 
         self.features, self._metadata_mtd_ft, self.groups = proc_feat
         del proc_feat
 
         self.summary, self._metadata_mtd_sm = _internal.process_summary(
             summary, wildcard=wildcard
-        )  # type: t.Tuple[t.Tuple[str, ...], _TypeSeqExt]
+        )
 
         self.timeopt = _internal.process_generic_option(
             value=measure_time, group_name="timeopt", allow_none=True
@@ -347,21 +354,19 @@ class MFE:
 
     def _call_summary_methods(
         self,
-        feature_values: t.Sequence[_internal.TypeNumeric],
+        feature_values: t.List[_internal.TypeNumeric],
         feature_name: str,
         verbose: int = 0,
         suppress_warnings: bool = False,
         **kwargs,
-    ) -> t.Tuple[
-        t.List[str], t.List[t.Union[float, t.Sequence]], t.List[float]
-    ]:
+    ) -> t.Tuple[t.List[str], t.List[t.Union[float, t.List]], t.List[float]]:
         """Invoke summary functions loaded in the model on given feature
         values.
 
         Parameters
         ----------
         feature_values : :obj:`sequence` of numerics
-            Sequence containing values from feature-extraction methods.
+            List containing values from feature-extraction methods.
 
         feature_name : :obj:`str`
             Name of the feature method used for produce the ``feature_value.``
@@ -411,7 +416,7 @@ class MFE:
                     by both ``mean`` and ``sd`` (standard deviation), giving
                     the values ``0.98347`` and ``0.34436``, respectively.
         """
-        metafeat_vals = []  # type: t.List[t.Union[int, float, t.Sequence]]
+        metafeat_vals = []  # type: t.List[t.Union[int, float, t.List]]
         metafeat_names = []  # type: t.List[str]
         metafeat_times = []  # type: t.List[float]
 
@@ -420,8 +425,8 @@ class MFE:
 
             if verbose >= 2:
                 print(
-                    " {} Summarizing '{}' feature with '{}' summary "
-                    "function...".format(
+                    " {} Summarizing '{}' feature with '{}' summary"
+                    " function...".format(
                         _internal.VERBOSE_BLOCK_MID_SYMBOL,
                         feature_name,
                         sm_mtd_name,
@@ -455,9 +460,9 @@ class MFE:
             if isinstance(summarized_val, np.ndarray):
                 summarized_val = summarized_val.flatten().tolist()
 
-            if isinstance(
-                summarized_val, collections.Sequence
-            ) and not isinstance(summarized_val, str):
+            if isinstance(summarized_val, list) and not isinstance(
+                summarized_val, str
+            ):
                 metafeat_vals += summarized_val
                 metafeat_names += [
                     ".".join((feature_name, sm_mtd_name, str(i)))
@@ -491,7 +496,9 @@ class MFE:
         suppress_warnings: bool = False,
         **kwargs,
     ) -> t.Tuple[
-        t.List[str], t.List[t.Union[int, float, t.Sequence]], t.List[float]
+        t.List[str],
+        t.List[t.Union[int, float, t.List]],
+        t.List[float],
     ]:
         """Invoke feature methods loaded in the model and gather results.
 
@@ -500,12 +507,17 @@ class MFE:
         For more information, check ``extract`` method documentation for
         in-depth information about arguments and return value.
         """
-        metafeat_vals = []  # type: t.List[t.Union[int, float, t.Sequence]]
+        metafeat_vals = []  # type: t.List[t.Union[int, float, t.List]]
         metafeat_names = []  # type: t.List[str]
         metafeat_times = []  # type: t.List[float]
 
         skipped_count = 0
-        for ind, cur_metadata in enumerate(self._metadata_mtd_ft, 1):
+
+        _iterator = enumerate(
+            tqdm.auto.tqdm(self._metadata_mtd_ft, disable=verbose != 1), 1
+        )
+
+        for ind, cur_metadata in _iterator:
             (
                 ft_mtd_name,
                 ft_mtd_callable,
@@ -556,12 +568,10 @@ class MFE:
                 suppress_warnings,
             )
 
-            ft_has_length = isinstance(
-                features, (np.ndarray, collections.Sequence)
-            )
+            ft_has_length = hasattr(features, "__len__")
 
             if ft_has_length and self._timeopt_type_is_avg():
-                time_ft /= len(features)
+                time_ft /= max(1, len(features))
 
             if self._metadata_mtd_sm and ft_has_length:
                 sm_ret = self._call_summary_methods(
@@ -583,14 +593,6 @@ class MFE:
                 metafeat_names.append(ft_name_without_prefix)
                 metafeat_times.append(time_ft)
 
-            if verbose > 0:
-                _internal.print_verbose_progress(
-                    cur_progress=100 * ind / len(self._metadata_mtd_ft),
-                    cur_mtf_name=ft_mtd_name,
-                    item_type="feature",
-                    verbose=verbose,
-                )
-
         if verbose == 1:
             _t_num_cols, _ = shutil.get_terminal_size()
             print(
@@ -644,7 +646,9 @@ class MFE:
         if self.X is None:
             raise TypeError("X can't be 'None'.")
 
-        categorical_cols = None  # type: np.ndarray[bool]
+        categorical_cols = (
+            None
+        )  # type: t.Optional[t.Union[t.List, np.ndarray]]
 
         if not cat_cols:
             categorical_cols = np.array([False] * self.X.shape[1])
@@ -666,10 +670,7 @@ class MFE:
                     arr=self.X,
                 )
 
-        elif isinstance(
-            cat_cols, (np.ndarray, collections.Iterable)
-        ) and not isinstance(cat_cols, str):
-            # and all(isinstance(x, int) for x in cat_cols)):
+        elif hasattr(cat_cols, "__len__") and not isinstance(cat_cols, str):
             categorical_cols = [i in cat_cols for i in range(self.X.shape[1])]
 
         else:
@@ -709,8 +710,8 @@ class MFE:
             Time necessary to extract some feature.
 
         times_sm : :obj:`list` of :obj:`float`
-            List of values to summarize the metafeature value with each summary
-            function.
+            List of values to summarize the metafeature value with each
+            summary function.
 
         Returns
         -------
@@ -865,8 +866,8 @@ class MFE:
             and transform_cat not in _internal.VALID_TRANSFORM_CAT
         ):
             raise ValueError(
-                "Invalid 'transform_cat' value ('{}'). Must be "
-                "a value in {}.".format(
+                "Invalid 'transform_cat' value ('{}'). "
+                "Must be a value in {}.".format(
                     transform_cat, _internal.VALID_TRANSFORM_CAT
                 )
             )
@@ -902,8 +903,8 @@ class MFE:
 
     def fit(
         self,
-        X: t.Sequence,
-        y: t.Optional[t.Sequence] = None,
+        X: t.Union[np.ndarray, t.List],
+        y: t.Optional[t.Union[np.ndarray, t.List]] = None,
         transform_num: bool = True,
         transform_cat: str = "gray",
         rescale: t.Optional[str] = None,
@@ -920,10 +921,10 @@ class MFE:
 
         Parameters
         ----------
-        X : :obj:`Sequence`
+        X : :obj:`List`
             Predictive attributes of the dataset.
 
-        y : :obj:`Sequence`, optional
+        y : :obj:`List`, optional
             Target attributes of the dataset, assuming that it is a supervised
             task.
 
@@ -1002,7 +1003,7 @@ class MFE:
             are the parameter names as strings and the values, the
             corresponding parameter value.
 
-        cat_cols :obj:`Sequence` of :obj:`int` or :obj:`str`, optional
+        cat_cols :obj:`List` of :obj:`int` or :obj:`str`, optional
             Categorical columns of dataset. If given :obj:`NoneType` or an
             empty sequence, assume all columns as numeric. If given value
             ``auto``, then an attempt of automatic detection is performed while
@@ -1029,7 +1030,7 @@ class MFE:
             Value used as ``select all`` for ``precomp_groups``.
 
         suppress_warnings : :obj:`bool`, optional
-            If True, ignore all warnings invoked while fitting dataset.
+            If True, ignore all warnings invoked while fitting the dataset.
 
         verbose : :obj:`int`, optional
             Defines the level of verbosity for the fit method. If `1`, then
@@ -1060,49 +1061,56 @@ class MFE:
             If X or y (or both) is neither a :obj:`list` or a :obj:`np.ndarray`
             object.
         """
-        if verbose >= 2:
-            print("Fitting data into model... ", end="")
+        with warnings.catch_warnings():
+            if suppress_warnings:
+                warnings.filterwarnings("ignore")
 
-        self.X, self.y = _internal.check_data(X, y)
+            if verbose >= 2:
+                print("Fitting data into model... ", end="")
 
-        if verbose >= 2:
-            print("Done.")
+            self.X, self.y = _internal.check_data(X, y)
 
-        rescale = _internal.process_generic_option(
-            value=rescale, group_name="rescale", allow_none=True
-        )
+            if verbose >= 2:
+                print("Done.")
 
-        self._fill_col_ind_by_type(cat_cols=cat_cols, check_bool=check_bool)
-
-        if verbose >= 2:
-            print(
-                "Started data transformation process.",
-                " {} Encoding numerical data into discrete values... ".format(
-                    _internal.VERBOSE_BLOCK_END_SYMBOL
-                ),
-                sep="\n",
-                end="",
+            rescale = _internal.process_generic_option(
+                value=rescale, group_name="rescale", allow_none=True
             )
 
-        data_cat = self._set_data_categoric(transform_num=transform_num)
-
-        if verbose >= 2:
-            print(
-                "Done.",
-                " {} Enconding categorical data into numerical values... "
-                .format(_internal.VERBOSE_BLOCK_END_SYMBOL),
-                sep="\n",
-                end="",
+            self._fill_col_ind_by_type(
+                cat_cols=cat_cols, check_bool=check_bool
             )
 
-        data_num = self._set_data_numeric(
-            transform_cat=transform_cat,
-            rescale=rescale,
-            rescale_args=rescale_args,
-        )
+            if verbose >= 2:
+                print(
+                    "Started data transformation process.",
+                    " {} Encoding numerical data into discrete values..."
+                    "".format(_internal.VERBOSE_BLOCK_END_SYMBOL),
+                    sep="\n",
+                    end="",
+                )
 
-        if verbose >= 2:
-            print("Done.", "Finished data transformation process.", sep="\n")
+            data_cat = self._set_data_categoric(transform_num=transform_num)
+
+            if verbose >= 2:
+                print(
+                    "Done.",
+                    " {} Enconding categorical data into numerical values..."
+                    "".format(_internal.VERBOSE_BLOCK_END_SYMBOL),
+                    sep="\n",
+                    end="",
+                )
+
+            data_num = self._set_data_numeric(
+                transform_cat=transform_cat,
+                rescale=rescale,
+                rescale_args=rescale_args,
+            )
+
+            if verbose >= 2:
+                print(
+                    "Done.", "Finished data transformation process.", sep="\n"
+                )
 
         # Custom arguments for metafeature extraction methods
         self._custom_args_ft = {
@@ -1126,15 +1134,20 @@ class MFE:
 
         _time_start = time.time()
 
-        # Custom arguments from preprocessing methods
-        self._precomp_args_ft = _internal.process_precomp_groups(
-            precomp_groups=precomp_groups,
-            groups=self.groups,
-            wildcard=wildcard,
-            suppress_warnings=suppress_warnings,
-            verbose=verbose,
-            **{**self._custom_args_ft, **kwargs},
-        )
+        with warnings.catch_warnings():
+            if suppress_warnings:
+                warnings.filterwarnings("ignore")
+
+            # Custom arguments from preprocessing methods
+            self._precomp_args_ft = _internal.process_precomp_groups(
+                precomp_groups=precomp_groups,
+                groups=self.groups,
+                wildcard=wildcard,
+                suppress_warnings=suppress_warnings,
+                verbose=verbose,
+                **self._custom_args_ft,
+                **kwargs,
+            )
 
         self.time_precomp = time.time() - _time_start
 
@@ -1168,8 +1181,9 @@ class MFE:
         verbose: int = 0,
         enable_parallel: bool = False,
         suppress_warnings: bool = False,
+        out_type: t.Any = tuple,
         **kwargs,
-    ) -> t.Tuple[t.Sequence, ...]:
+    ) -> _TypeExtract:
         """Extracts metafeatures from the previously fitted dataset.
 
         Parameters
@@ -1189,10 +1203,7 @@ class MFE:
             (to be implemented).
 
         suppress_warnings : :obj:`bool`, optional
-            If True, do not show warnings about unknown user custom parameters
-            for feature extraction and summary methods passed via kwargs. Note
-            that both feature extraction and summary methods may still raise
-            warnings by itself.
+            If True, do not show any warning while extracting meta-features.
 
         kwargs:
             Used to pass custom arguments for both feature-extraction and
@@ -1208,10 +1219,16 @@ class MFE:
 
             For more information see Examples.
 
+        out_type:  :obj:`Any`, optional
+            If tuple, then the returned value is a tuple. If dict, then the
+            returned value is a dictionary. If pd.DataFrame the the returned
+            value is a pandas.core.DataFrame. Otherwise, an Type Error
+            is raised.
+
         Returns
         -------
         :obj:`tuple`(:obj:`list`, :obj:`list`)
-            A tuple containing two lists (if ``measure_time`` is None.)
+            A tuple containing two lists (if ``measure_time`` is None).
 
             The first field is the identifiers of each summarized value in the
             form ``feature_name.summary_mtd_name`` (i.e., the feature
@@ -1224,11 +1241,32 @@ class MFE:
             (i.e., the value at index ``i`` in the second list has its
             identifier at the same index in the first list and vice-versa).
 
-            Example:
-                ([``attr_ent.mean``, ``attr_ent.sd``], [``0.983``, ``0.344``])
-                is the return value for the feature ``attr_end`` summarized by
-                both ``mean`` and ``sd`` (standard deviation), giving the valu-
-                es ``0.983`` and ``0.344``, respectively.
+        :obj:`dict`(:obj:`str`, :obj:`list`)
+            A dictionary containing two fields (if ``measure_time`` is None).
+            The fields are: `mtf_names`, `mtf_vals` (if ``measure_time``, the
+            there is `mtf_time`).
+
+            The first field is the identifiers of each summarized value in the
+            form ``feature_name.summary_mtd_name`` (i.e., the feature
+            extraction name concatenated by the summary method name, separated
+            by a dot).
+
+            The second field is the summarized values.
+
+            Both lists of each field have a 1-1 correspondence by the index of
+            each elemen (i.e., the value at index ``i`` in the second list has
+            its identifier at the same index in the first list and vice-versa).
+
+        :obj:`pandas.core.frame.DataFrame`
+            A pandas DataFrame instance.
+
+            Each column is a summarized value. The column is identified by the
+            name of the meta-feature in the form
+            ``feature_name.summary_mtd_name`` (i.e., the featur extraction name
+            concatenated by the summary method name, separate by a dot).
+
+            The rows store the summarized values (if ``measure_time``, there
+            is a row with the time taken to calculate each value).
 
             if ``measure_time`` is given during the model instantiation, a
             third list will be returned with the time spent during the
@@ -1239,6 +1277,9 @@ class MFE:
         TypeError
             If calling ``extract`` method before ``fit`` method.
 
+        TypeError
+            If calling ``extract`` method with invalid ``out_type``.
+
         Examples
         --------
         Using kwargs. Option 1 to pass ft. extraction custom arguments:
@@ -1274,20 +1315,24 @@ class MFE:
 
         _time_start = time.time()
 
-        results = self._call_feature_methods(
-            verbose=verbose,
-            enable_parallel=enable_parallel,
-            suppress_warnings=suppress_warnings,
-            **kwargs,
-        )  # type: t.Tuple[t.List, ...]
+        with warnings.catch_warnings():
+            if suppress_warnings:
+                warnings.filterwarnings("ignore")
 
-        _internal.post_processing(
-            results=results,
-            groups=self.groups,
-            suppress_warnings=suppress_warnings,
-            **self._postprocess_args_ft,
-            **kwargs,
-        )
+            results = self._call_feature_methods(
+                verbose=verbose,
+                enable_parallel=enable_parallel,
+                suppress_warnings=suppress_warnings,
+                **kwargs,
+            )  # type: t.Tuple[t.List, ...]
+
+            _internal.post_processing(
+                results=results,
+                groups=self.groups,
+                suppress_warnings=suppress_warnings,
+                **self._postprocess_args_ft,
+                **kwargs,
+            )
 
         self.time_extract = time.time() - _time_start
         self.time_total = self.time_extract + self.time_precomp
@@ -1328,124 +1373,176 @@ class MFE:
                 sep="\n",
             )
 
-        if self.timeopt:
-            return res_names, res_vals, res_times
+        _deal_types = {
+            tuple: lambda names, vals, times=[]: (names, vals, times)
+            if self.timeopt
+            else (names, vals),
+            dict: lambda names, vals, times=[]: {
+                "mtf_names": names,
+                "mtf_vals": vals,
+                "mtf_time": times,
+            }
+            if self.timeopt
+            else {"mtf_names": names, "mtf_vals": vals},
+            pd.DataFrame: lambda names, vals, times=[]: pd.DataFrame(
+                data=(vals, times), columns=names, index=("values", "time")
+            )
+            if self.timeopt
+            else pd.DataFrame(data=(vals,), columns=names),
+        }
 
-        return res_names, res_vals
+        try:
+            return _deal_types[out_type](res_names, res_vals, res_times)
+        except KeyError as out_not_defined:
+            raise TypeError("Output type not supported.") from out_not_defined
 
-    def _extract_with_bootstrap(
-        self,
-        extractor: "MFE",
-        sample_num: int,
-        arguments_fit: t.Dict[str, t.Any],
-        arguments_extract: t.Dict[str, t.Any],
-        verbose: int = 0,
-    ) -> t.Tuple[np.ndarray, ...]:
-        """Extract metafeatures using bootstrapping."""
-        if self.X is None:
-            raise TypeError(
-                "Fitted data not found. Please call 'fit' method first."
+    def extract_metafeature_names(
+        self, supervised: bool = True
+    ) -> t.Tuple[str, ...]:
+        """Extract the pre-configured meta-feature names.
+
+        Parameters
+        ----------
+        supervised : bool, optional
+            If True, extract the meta-feature names assuming that `y` (data
+            labels) is given alongside `X` (independent attributes).
+
+            If there is some data fit into the MFE model, this method checks
+            wether `y` was fitted or not. Therefore, setting `supervised=True`
+            while fitting only `X` has no effect, and only unsupervised
+            meta-feature names will be returned.
+
+        Returns
+        -------
+        tuple
+            If Tuple with meta-feature names to be extracted as values.
+
+        """
+        if self.X is not None:
+            custom_args_ft = self._custom_args_ft
+            precomp_args_ft = self._precomp_args_ft
+            postprocess_args_ft = self._postprocess_args_ft
+
+        else:
+            # No data fit, assume given arguments.
+            given_arguments = {
+                "X",
+                "N",
+                "C",
+                "num_cv_folds",
+                "shuffle_cv_folds",
+                "lm_sample_frac",
+                "score",
+                "random_state",
+                "cat_cols",
+                "hypparam_model_dt",
+            }
+
+            if supervised:
+                given_arguments.add("y")
+                given_arguments.add("dt_model")
+
+            postprocess_args_ft = {
+                "inserted_group_dep": self.inserted_group_dep,
+            }
+
+            custom_args_ft = dict.fromkeys(given_arguments, None)
+            precomp_args_ft = {}
+
+        metafeat_names = []  # type: t.List[str]
+
+        for cur_metadata in self._metadata_mtd_ft:
+            (
+                ft_mtd_name,
+                ft_mtd_callable,
+                ft_mtd_args,
+                ft_mandatory,
+            ) = cur_metadata
+
+            ft_name_without_prefix = _internal.remove_prefix(
+                value=ft_mtd_name, prefix=_internal.MTF_PREFIX
             )
 
-        def _handle_extract_ret(
-            res: t.Tuple[np.ndarray, ...],
-            args: t.Tuple[t.Sequence, ...],
-            it_num: int,
-        ) -> t.Tuple[np.ndarray, ...]:
-            """Handle each .extraction method return value."""
-            mtf_names, mtf_vals, mtf_time = res
+            try:
+                # Test if meta-feature can be extracted.
+                _internal.build_mtd_kwargs(
+                    mtd_name=ft_name_without_prefix,
+                    mtd_args=ft_mtd_args,
+                    mtd_mandatory=ft_mandatory,
+                    user_custom_args=None,
+                    inner_custom_args=custom_args_ft,
+                    precomp_args=precomp_args_ft,
+                    suppress_warnings=True,
+                )
 
-            if not self.timeopt:
-                cur_mtf_names, cur_mtf_vals = args
+            except RuntimeError:
+                continue
+
+            ft_has_length = _internal.array_is_returned(ft_mtd_callable)
+
+            if self._metadata_mtd_sm and ft_has_length:
+                for cur_metadata_sm in self._metadata_mtd_sm:
+                    sm_mtd_name, sm_mtd_callable, _, _ = cur_metadata_sm
+
+                    try:
+                        summarized_val_len = len(sm_mtd_callable([0]))
+
+                    except TypeError:
+                        summarized_val_len = 0
+
+                    if summarized_val_len > 0:
+                        metafeat_names += [
+                            ".".join(
+                                (ft_name_without_prefix, sm_mtd_name, str(i))
+                            )
+                            for i in range(summarized_val_len)
+                        ]
+
+                    else:
+                        metafeat_names.append(
+                            ".".join((ft_name_without_prefix, sm_mtd_name))
+                        )
 
             else:
-                cur_mtf_names, cur_mtf_vals, cur_mtf_time = args
+                metafeat_names.append(ft_name_without_prefix)
 
-            if mtf_names.size:
-                mtf_vals[:, it_num] = cur_mtf_vals
+        fake_vals = len(metafeat_names) * [0]
 
-                if self.timeopt:
-                    mtf_time += cur_mtf_time
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
 
-            else:
-                mtf_names = np.asarray(cur_mtf_names, dtype=str)
-                mtf_vals = np.zeros(
-                    (len(cur_mtf_vals), sample_num), dtype=float
-                )
-                mtf_vals[:, 0] = cur_mtf_vals
-
-                if self.timeopt:
-                    mtf_time = np.asarray(cur_mtf_time, dtype=float)
-
-            return mtf_names, mtf_vals, mtf_time
-
-        res = 3 * (np.array([]),)
-
-        if self.random_state is None:
-            # Enforce pseudo-random behaviour to avoid previously set
-            # random seeds out of this context
-            np.random.seed()
-
-        bootstrap_random_state = (
-            self.random_state
-            if self.random_state is not None
-            else np.random.randint(2 ** 20 - 1)
-        )
-
-        for it_num in np.arange(sample_num):
-            if verbose > 0:
-                print(
-                    "Extracting from sample dataset {} of {} ({:.2f}%)..."
-                    .format(
-                        1 + it_num,
-                        sample_num,
-                        100.0 * (1 + it_num) / sample_num,
-                    )
-                )
-
-            # Note: setting random state to prevent same sample indices due
-            # to random states set during fit/extraction
-            np.random.seed(bootstrap_random_state)
-            bootstrap_random_state += 1
-
-            sample_inds = np.random.randint(
-                self.X.shape[0], size=self.X.shape[0]
+            _internal.post_processing(
+                results=(metafeat_names, fake_vals, fake_vals),
+                groups=self.groups,
+                suppress_warnings=True,
+                **postprocess_args_ft,
             )
 
-            X_sample = self.X[sample_inds, :]
-            y_sample = self.y[sample_inds] if self.y is not None else None
-
-            extractor.fit(X_sample, y_sample, **arguments_fit)
-
-            res = _handle_extract_ret(
-                res=res,
-                args=extractor.extract(**arguments_extract),
-                it_num=it_num,
-            )
-
-            if verbose > 0:
-                print(
-                    "Done extracting from sample dataset {}.\n".format(
-                        1 + it_num
-                    )
-                )
-
-        return res
+        return tuple(sorted(metafeat_names))
 
     def extract_with_confidence(
         self,
         sample_num: int = 128,
-        confidence: t.Union[float, t.Sequence[float]] = 0.95,
-        return_avg_val: bool = True,
+        confidence: t.Union[float, t.List[float]] = 0.95,
         arguments_fit: t.Optional[t.Dict[str, t.Any]] = None,
         arguments_extract: t.Optional[t.Dict[str, t.Any]] = None,
         verbose: int = 0,
-    ) -> t.Tuple[t.List, ...]:
+    ) -> _TypeExtract:
         """Extract metafeatures with confidence intervals.
 
-        To build the confidence intervals, each metafeature is extracted
-        ``sample_num`` times from a distinct dataset built from the
-        fitted data using bootstrap.
+        To build the confidence intervals, the empirical bootstrap algorithm
+        is used, which is as follows:
+
+        1. All selected metafeatures are extracted from the fitted data, `M`.
+        2. Then, each metafeature is extracted ``sample_num`` times from a
+           resampled dataset using bootstrap from the fitted data, `M_i`.
+        3. Then, the differences `delta_i` = `M_i` - `M` are calculated
+        4. From the differences `delta_i`, the quantiles related to the given
+           confidence levels (confidence = 1 - Type I error rate) are
+           calculated.
+        5. The confidence intervals are centered in `M` and the width of
+           interval is given by the quantiles of the differences previously
+           calculated.
 
         All configuration used by this method are from the configuration
         while instantiating the current model.
@@ -1459,19 +1556,7 @@ class MFE:
         confidence : float or sequence of floats, optional
             Confidence level of the interval. Must be in (0.0, 1.0) range.
             If a sequence of confidence levels is given, a confidence
-            interval will be extracted for all values. Each confidence
-            interval will be calculated as [confidence/2, 1 - confidence/2].
-
-        return_avg_vals : bool, optional
-            If True, return the average value for both the metafeature
-            values and the time elapsed for its extraction (if any
-            ``measure_time`` option was chosen.) If False, then all
-            extracted metafeature values are returned as a 2D numpy array
-            of shape (`metafeature_num`, `sample_num`) (i.e., each row
-            represents a distinct metafeature, and each column is the
-            value of the corresponding metafeature extracted from a
-            distinct sample dataset) and the time elapsed will be the
-            sum of all extractions for each metafeature.
+            interval will be extracted for each value.
 
         arguments_fit : dict, optional
             Extra arguments for the fit method for each sampled dataset.
@@ -1507,66 +1592,34 @@ class MFE:
             (lower_0.80, lower_0.90, lower_0.99, upper_0.80, upper_0.90,
             upper_0.99).
 
-            if ``return_avg_val`` is True, the metafeature values and the
-            time elapsed for extraction for each item (if any ``measure_time``
-            options was chosen) will be the average value between all
-            extractions. Otherwise, all extracted metafeature values will be
-            returned as a 2D numpy array (where each columns is from a distinct
-            sampled dataset, and each row is a distinct metafeature), and the
-            time elapsed will be the sum of all extractions for the
-            corresponding metafeature.
-
         Raises
         ------
         ValueError
             If ``confidence`` is not in (0.0, 1.0) range.
 
+        TypeError
+            If no data was fit into the model previously.
+
         Notes
         -----
         The model used to fit and extract metafeatures for each sampled
         dataset is instantiated within this method and, therefore, this
         method does not affect the current model (if any) by any means.
         """
-        _confidence = np.asarray(confidence, dtype=float)
-
-        if np.any(np.logical_or(_confidence <= 0.0, _confidence >= 1.0)):
-            raise ValueError(
-                "'_confidence' must be in (0.0, 1.0) range (got {}.)".format(
-                    _confidence
-                )
+        if not isinstance(self.X, np.ndarray):
+            raise TypeError(
+                "Data not found. Please use MFE.fit() method "
+                "before MFE.extract_with_confidence()."
             )
 
         if self.random_state is not None:
             np.random.seed(self.random_state)
 
-        if arguments_fit is None:
-            arguments_fit = {}
-
-        if arguments_extract is None:
-            arguments_extract = {}
-
         # Note: the metafeature extraction random seed will be fixed due
         # to the random indices while bootstrapping the fitted data.
         _random_state = self.random_state if self.random_state else 1234
 
-        if verbose > 0:
-            print("Started metafeature extract with _confidence interval.")
-            print("Random seed:")
-            print(
-                " {} For extractor model: {}{}".format(
-                    _internal.VERBOSE_BLOCK_END_SYMBOL,
-                    _random_state,
-                    "" if self.random_state else " (chosen by default)",
-                )
-            )
-
-            print(
-                " {} For bootstrapping: {}".format(
-                    _internal.VERBOSE_BLOCK_END_SYMBOL, self.random_state
-                )
-            )
-
-        extractor = MFE(
+        surrogate_extractor = MFE(
             features=self.features,
             groups=self.groups,
             summary=self.summary,
@@ -1574,35 +1627,56 @@ class MFE:
             random_state=_random_state,
         )
 
-        mtf_names, mtf_vals, mtf_time = self._extract_with_bootstrap(
-            extractor=extractor,
+        bootstrap_extractor = _bootstrap.BootstrapExtractor(
+            extractor=surrogate_extractor,
             sample_num=sample_num,
-            verbose=verbose,
+            confidence=confidence,
             arguments_fit=arguments_fit,
             arguments_extract=arguments_extract,
+            verbose=verbose,
+            random_state=self.random_state,
         )
 
-        if verbose > 0:
-            print("Finished metafeature extract with _confidence interval.")
-            print("Now getting _confidence intervals...", end=" ")
+        bootstrap_extractor.fit(self.X, self.y)
 
-        _half_sig_level = 0.5 * (1.0 - _confidence)
-        quantiles = np.hstack((_half_sig_level, 1.0 - _half_sig_level))
-        mtf_conf_int = np.quantile(a=mtf_vals, q=quantiles, axis=1).T
+        (
+            mtf_names,
+            mtf_vals,
+            mtf_time,
+            mtf_conf_int,
+        ) = bootstrap_extractor.extract_with_confidence()
 
-        if verbose > 0:
-            print("Done.")
+        _deal_types = {
+            tuple: lambda names, vals, conf, times=[]: (
+                names,
+                vals,
+                times,
+                conf,
+            )
+            if self.timeopt
+            else (names, vals, conf),
+            dict: lambda names, vals, conf, times=[]: {
+                "mtf_names": names,
+                "mtf_vals": vals,
+                "confidence": conf,
+                "mtf_time": times,
+            }
+            if self.timeopt
+            else {"mtf_names": names, "mtf_vals": vals, "confidence": conf},
+        }
 
-        if return_avg_val:
-            mtf_vals = np.nanmean(mtf_vals, axis=1)
+        # Check if the type was defined previously
+        if arguments_extract and "out_type" in arguments_extract:
+            out_type = arguments_extract["out_type"]
+        else:
+            out_type = tuple
 
-        if self.timeopt:
-            if return_avg_val:
-                mtf_time /= sample_num
-
-            return mtf_names, mtf_vals, mtf_time, mtf_conf_int
-
-        return mtf_names, mtf_vals, mtf_conf_int
+        try:
+            return _deal_types[out_type](
+                mtf_names, mtf_vals, mtf_conf_int, mtf_time
+            )
+        except KeyError as out_not_defined:
+            raise TypeError("Unknown output type.") from out_not_defined
 
     def extract_from_model(
         self,
@@ -1610,7 +1684,7 @@ class MFE:
         arguments_fit: t.Optional[t.Dict[str, t.Any]] = None,
         arguments_extract: t.Optional[t.Dict[str, t.Any]] = None,
         verbose: int = 0,
-    ) -> t.Tuple[t.Sequence, ...]:
+    ) -> _TypeExtract:
         """Extract model-based metafeatures from given model.
 
         The random seed used by the new internal model is the same random
@@ -1644,7 +1718,9 @@ class MFE:
 
         Returns
         -------
-        :obj:`tuple`(:obj:`list`, :obj:`list`)
+        :obj:`tuple`(:obj:`list`, :obj:`list`) or
+        :obj:`dict`(:obj:`str`, :obj:`any`) or
+        :obj:`pandas.core.DataFrame`
             See `.extract` method return value for more information.
 
         Notes
@@ -1675,12 +1751,12 @@ class MFE:
         try:
             sklearn.utils.validation.check_is_fitted(model)
 
-        except sklearn.exceptions.NotFittedError:
+        except sklearn.exceptions.NotFittedError as err:
             raise RuntimeError(
                 "Given 'model' does not have any fitted data. "
                 "Please use its 'fit' method before using the "
                 "model with 'extract_from_model' method."
-            )
+            ) from err
 
         if arguments_fit is None:
             arguments_fit = {}
@@ -1711,7 +1787,7 @@ class MFE:
 
             print(
                 "Total of {} 'model-based' metafeature method candidates."
-                .format(len(_fts))
+                "".format(len(_fts))
             )
 
             print("Started extraction from model.")
@@ -1785,13 +1861,14 @@ class MFE:
 
     @classmethod
     def valid_metafeatures(
-        cls, groups: t.Optional[t.Union[str, t.Iterable[str]]] = None,
+        cls,
+        groups: t.Optional[t.Union[str, t.Iterable[str]]] = None,
     ) -> t.Tuple[str, ...]:
         """Return a tuple with all metafeatures related to given ``groups``.
 
         Parameters
         ----------
-        groups : :obj:`Sequence` of :obj:`str` or :obj:`str`, optional:
+        groups : :obj:`List` of :obj:`str` or :obj:`str`, optional:
             Can be a string such value is a name of a specific metafeature
             group (see ``valid_groups`` method for more information) or a
             sequence of metafeature group names. It can be also None, which
@@ -1830,8 +1907,8 @@ class MFE:
     @classmethod
     def parse_by_group(
         cls,
-        groups: t.Union[t.Sequence[str], str],
-        extracted_results: t.Tuple[t.Sequence, ...],
+        groups: t.Union[t.List[str], str],
+        extracted_results: t.Tuple[t.List, ...],
     ) -> t.Tuple[t.List, ...]:
         """Parse the result of ``extract`` for given metafeature ``groups``.
 
@@ -1840,12 +1917,12 @@ class MFE:
 
         Parameters
         ----------
-        groups : :obj:`Sequence` of :obj:`str` or :obj:`str`
+        groups : :obj:`List` of :obj:`str` or :obj:`str`
             Metafeature group names which the results should be parsed
             relative to. Use ``valid_groups`` method to check the available
             metafeature groups.
 
-        extracted_results : :obj:`tuple` of :obj:`t.Sequence`
+        extracted_results : :obj:`tuple` of :obj:`t.List`
             Output of ``extract`` method. Should contain all outputed lists
             (metafeature names, values and elapsed time for extraction, if
             present.)
